﻿// ShelfLabelCreator
//
// Copyright (C) 2012 CASIO Europe GmbH
//
// IT9000 ShelfLabelCreator provided AS IS for educational purposes only,
// with no warranties, and confers no rights. Error checking is 
// elided for clarity. Carry out sufficient tests prior to using
// it in a productive environment.
// CASIO Europe GmbH accepts no responsibility for loss or damage
// arising from its use, including damage from virus.
//
// Please, note: This configuration information may be changed
// without any further notice.
// Casio Europe GmbH does not guarantee future compatibility. By 
// using this software you agree to these terms and conditions.
///////////////////////////////////////////////////////////////////

using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;
using System.Globalization;
using Calib; //Casio Library

namespace ShelfLabelCreator
{
    public partial class MainForm : Form
    {
       
        #region Fields

        private static int TriggerLeft;
        private static int TriggerRight;
        private static int TriggerCenter;
        private const int SystemLibrarySuccess = -1;
        private const int ImagerTimeout = 10000;
        // Increase the size if necessary:
        private const int BufferSize = 0xFF; 

        #endregion

        #region Trigger Key Handling

        /// <summary>
        /// Template for callback function.
        /// </summary>
        delegate int KeyStateDelegate();
        /// <summary>
        /// Callback function for IMGWaitForDecode.
        /// </summary>
        KeyStateDelegate keyStateDelegate = new KeyStateDelegate(OnKeyState);

        /// <summary>
        /// Called by IMGWaitForDecode in MainForm_KeyDown.
        /// </summary>
        /// <returns>The key state.</returns>
        /// <remarks>Change or add more keys here.</remarks>
        /// 
    
        public static int OnKeyState()
        {
            if (NativeMethods.GetAsyncKeyState(TriggerLeft) < 0 ||
                NativeMethods.GetAsyncKeyState(TriggerRight) < 0 ||
                NativeMethods.GetAsyncKeyState(TriggerCenter) < 0)
            {
                return 1;   // Key is held down
            }
            else
            {
                return 0;   // Key is released
            }
        }

        #endregion
                
        public MainForm()
        {
            InitializeComponent();

            #region resources
            Resources.Culture = CultureInfo.CurrentCulture;
            this.button_create_shelflabel.Text = Resources.Button_Print;
            this.button_new.Text = Resources.Button_Clear;
            this.label_barcode.Text = Resources.Label_Barcode;
            this.label_item.Text = Resources.Label_Item;
            this.label_price.Text = Resources.Label_Price;
            this.label_shelflabel.Text = Resources.Label_Tag;
            this.checkBox_enable_barcode.Text = Resources.Checkbox_Barcode;
            #endregion


            //Get Model ID to disable mainMenu in Windws CE 
            int dwModel = 0;
			int dwVersion = 0;
			int dwPlatform = 0;
            if (SystemLibNet.Api.SysGetModelName(ref dwModel, ref dwVersion, ref dwPlatform) != 0)
            {
                if (dwModel == 19)	// IT-9000 Windows CE = 19
                 this.Menu = null;
            } 
 
            TriggerLeft = GetDefaultKey(SystemLibNet.Def.KEYID_LEFTTRIGGER);
            TriggerRight = GetDefaultKey(SystemLibNet.Def.KEYID_RIGHTTRIGGER);
            TriggerCenter = GetDefaultKey(SystemLibNet.Def.KEYID_CENTERTRIGGER);
           
            this.lbl_barcode.Visible = false;
            DateTime zeit = DateTime.Now;
            this.lbl_date.Text = zeit.ToShortDateString();

            OpenImager();
        }

        #region events
        //Deinit Imager on Application close
        private void MainForm_Closing(object sender, CancelEventArgs e)
        {
            CloseImager();
        }

        //Scan Barcode when Triggerkey is pressed
        private void MainForm_KeyDown(object sender, KeyEventArgs e)
        {
            if ((e.KeyValue == TriggerLeft ||
                e.KeyValue == TriggerRight ||
                e.KeyValue == TriggerCenter) &&
                e.KeyValue != -1)
            {
                int result;
                string pMessage = new string(' ', BufferSize);
                string pCodeID = new string(' ', 2);
                string pAimID = new string(' ', 3);
                string pSymModifier = new string(' ', 3);
                int pLength = 0;

                result = IMGLibNet.Api.IMGWaitForDecode(
                    ImagerTimeout,
                    pMessage,
                    pCodeID,
                    pAimID,
                    pSymModifier,
                    ref pLength,
                    Marshal.GetFunctionPointerForDelegate(keyStateDelegate));

                if (result == IMGLibNet.Def.IMG_SUCCESS && pLength > 0)
                {
                    this.textBox_barcode.Text = pMessage.Trim();
                }

                NativeMethods.MSG peekMsg = new NativeMethods.MSG();
                do
                {
                    if (NativeMethods.PeekMessage(
                        peekMsg,
                        IntPtr.Zero,
                        NativeMethods.WM_KEYFIRST,
                        NativeMethods.WM_KEYLAST,
                        NativeMethods.PM_REMOVE) == 0)
                    {
                        break;
                    }

                } while (true);
            }
        }

        //Clear up for new shelflabel
        private void button_new_Click(object sender, EventArgs e)
        {
            this.textBox_barcode.Text = "";
            this.lbl_barcode.Text = "";
            this.textBox_item.Text = "";
            this.lbl_item.Text = "";
            this.textBox_price.Text = "";
            this.lbl_price.Text = "";
            DateTime zeit = DateTime.Now;
            this.lbl_date.Text = zeit.ToShortDateString();
         }
        
        //Print barcode or only Codenumber
        private void checkBox_enable_barcode_CheckStateChanged(object sender, EventArgs e)
        {
            if (this.checkBox_enable_barcode.Checked == true)
            {
                this.pictureBox_barcode.Visible = true;
                this.lbl_barcode.Visible = false;
            }
            else
            {
                this.pictureBox_barcode.Visible = false;
                this.lbl_barcode.Visible = true;
            }
        }

        //Copy barode data to label
        private void textBox_barcode_TextChanged(object sender, EventArgs e)
        {
            this.lbl_barcode.Text = this.textBox_barcode.Text;
        }

        //Copy item data to label
        private void textBox_item_TextChanged(object sender, EventArgs e)
        {
            this.lbl_item.Text = this.textBox_item.Text;
        }

        //Copy price data to label
        private void textBox_price_TextChanged(object sender, EventArgs e)
        {
            this.lbl_price.Text = this.textBox_price.Text;
        }
        
        //Print Shelflabel
        private void button_create_shelflabel_Click(object sender, EventArgs e)
        {
            string code = this.textBox_barcode.Text.PadLeft(13, '0');
            IntPtr hWnd = this.panel_selflabel.Handle;

            if (Calib.PrinterLibNet.Api.PRNOpen() != Calib.PrinterLibNet.Def.PRN_NORMAL)
            {
                MessageBox.Show("PRN OPEN FAILURE", "STATUS", MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
            }

            if (Calib.PrinterLibNet.Api.PRNPrintWindow(hWnd) != Calib.PrinterLibNet.Def.PRN_NORMAL)
            {
                MessageBox.Show("PRN SELF PRINT FAILURE", "STATUS", MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
            }

            if (this.checkBox_enable_barcode.Checked == true)
            {
                this.pictureBox_barcode.Visible = true;

                if (Calib.PrinterLibNet.Api.PRNBarcodeOut(
                    0,				// JAN
                    10,				// Height
                    1,				// With check digit
                    1,				// With chars
                    0xFFFF,			// Centering mode
                    0,				// Standard direction
                    code.Length,	// Without check digit
                    code)			// Data
                   != Calib.PrinterLibNet.Def.PRN_NORMAL)
                {
                    MessageBox.Show("PRN BARCODE PRINT FAILURE", "STATUS", MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
                }

            }
            Calib.PrinterLibNet.Api.PRNTextOut(1, "\n");	// Feed...
            Calib.PrinterLibNet.Api.PRNTextOut(1, "\n");
            Calib.PrinterLibNet.Api.PRNTextOut(1, "\n");
            Calib.PrinterLibNet.Api.PRNTextOut(1, "\n");

            if (Calib.PrinterLibNet.Api.PRNClose() != Calib.PrinterLibNet.Def.PRN_NORMAL)
            {
                MessageBox.Show("PRN ClOSE FAILURE", "STATUS", MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
            }

        }
        #endregion

        #region Settings

        /// <summary>
        /// Makes particular settings.
        /// </summary>
        /// <returns>Success of operation as a return code from IMGLibNet.</returns>
        /// <remarks>In this sample EAN8 is just enabled,
        /// while all other settings for that symbology are conserved.</remarks>
        private int MakeParticularSettings()
        {
            int result;
            bool bEnabled = false;
            bool bXmitCheckChar = false;
            bool bAddendaReq = false;
            bool bAddendaSeperator = false;
            bool bAddenda2Digit = false;
            bool bAddendenda5Digit = false;

            result = IMGLibNet.Api.IMGGetEAN13(
                ref bEnabled,
                ref bXmitCheckChar,
                ref bAddendaReq,
                ref bAddendaSeperator,
                ref bAddenda2Digit,
                ref bAddendenda5Digit);

            if (result == IMGLibNet.Def.IMG_SUCCESS)
            {
                result = IMGLibNet.Api.IMGSetEAN13(
                    true,
                    bXmitCheckChar,
                    bAddendaReq,
                    bAddendaSeperator,
                    bAddenda2Digit,
                    bAddendenda5Digit);
            }

            return result;
        }

        #endregion

        #region System Library

        /// <summary>
        /// Gets the key value for a given key.
        /// </summary>
        /// <returns>The key value if successful; otherwise -1.</returns>
        private int GetDefaultKey(int keyId)
        {
            int defaultKey = -1;
            int[] buffer = new int[16];

            if (SystemLibNet.Api.SysGetDefaultKey(keyId, buffer) == SystemLibrarySuccess)
            {
                defaultKey = buffer[0] & 0xFF;
            }

            return defaultKey;
        }

        /// <summary>
        /// Gets the ini file path.
        /// </summary>
        /// <returns>The full path to the ini file.</returns>
        private string GetIniFilePath()
        {
            int result;
            char[] buffer = new char[256];
            string path = "";

            result = SystemLibNet.Api.SysGetSystemName(
                SystemLibNet.Def.IMGIni_FilePath, buffer);

            if (result == SystemLibrarySuccess)
            {
                path = new string(buffer);
                path = path.TrimEnd('\0');
            }

            return path;
        }

        #endregion

        #region Open Imager

        //Opens the imager.
        private void OpenImager()
        {
            if (Calib.IMGLibNet.Api.IMGInit() != IMGLibNet.Def.IMG_SUCCESS)
            {
                MessageBox.Show("IMAGER INIT FAILURE", "STATUS", MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
            }

            string path = GetIniFilePath();

            if (File.Exists(path) == true)
            {
                int result = IMGLibNet.Api.IMGLoadConfigFile(path);
            }

            if (MakeParticularSettings() != 0)
            {
                MessageBox.Show("IMAGER SETTINGS FAILURE", "STATUS", MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
            }

            if (Calib.IMGLibNet.Api.IMGConnect() != IMGLibNet.Def.IMG_SUCCESS)
            {
                MessageBox.Show("IMAGER CONNECT FAILURE", "STATUS", MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
            }
        }
        #endregion
        
        #region Close Imager

        // Closes the imager.
        private void CloseImager()
        {
            if (Calib.IMGLibNet.Api.IMGDisconnect() != IMGLibNet.Def.IMG_SUCCESS)
            {
                MessageBox.Show("IMAGER DISCONNECT FAILURE", "STATUS", MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
            }

            if (Calib.IMGLibNet.Api.IMGDeinit() != IMGLibNet.Def.IMG_SUCCESS)
            {
                MessageBox.Show("IMAGER DEINIT FAILURE", "STATUS", MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
            }
        }
        #endregion
        
        #region Get Code Type

        /// <summary>
        /// Gets the type.
        /// </summary>
        /// <param name="pCodeID">The code ID.</param>
        /// <returns>Code type.</returns>
        private string GetType(string pCodeID)
        {
            string type;

            switch (pCodeID)
            {
                case "A ":
                    type = "Australian Postal";
                    break;
                case "z ":
                    type = "Aztec";
                    break;
                case "B ":
                    type = "British Postal";
                    break;
                case "C ":
                    type = "Canadian Postal";
                    break;
                case "a ":
                    type = "Codabar";
                    break;
                case "q ":
                    type = "Codablock F";
                    break;
                case "h ":
                    type = "Code 11";
                    break;
                case "j ":
                    type = "Code 128 or ISBT";
                    break;
                case "b ":
                    type = "Code 39";
                    break;
                case "l ":
                    type = "Code 49 or EAN 128";
                    break;
                case "i ":
                    type = "Code 93";
                    break;
                case "y ":
                    type = "RSS or UCC/EAN Composite";
                    break;
                case "w ":
                    type = "DataMatrix";
                    break;
                case "K ":
                    type = "Dutch Postal";
                    break;
                case "d ":
                    type = "EAN13";
                    break;
                case "D ":
                    type = "EAN8";
                    break;
                case "f ":
                    type = "IATA 2of5";
                    break;
                case "e ":
                    type = "ITF(Interleaved 2of5)";
                    break;
                case "J ":
                    type = "Japanese Postal";
                    break;
                case "x ":
                    type = "Maxicode";
                    break;
                case "R ":
                    type = "MicroPDF";
                    break;
                case "g ":
                    type = "MSI";
                    break;
                case "r ":
                    type = "PDF417";
                    break;
                case "L ":
                    type = "Planet Code";
                    break;
                case "P ":
                    type = "Postnet";
                    break;
                case "O ":
                    type = "OCR";
                    break;
                case "s ":
                    type = "QR";
                    break;
                case "T ":
                    type = "TLC39";
                    break;
                case "c ":
                    type = "UPC Version A";
                    break;
                case "E ":
                    type = "UPC Version E0,E1";
                    break;
                default:
                    type = "Unknown Symbol";
                    break;
            }

            return type;
        }

        #endregion
        
        #region NativeMethods

        internal sealed class NativeMethods
        {

            private NativeMethods()
            {
            }

            internal const int WM_KEYFIRST = 0x0100;
            internal const int WM_KEYLAST = 0x0108;
            internal const int PM_REMOVE = 0x0001;

            internal class MSG
            {
                private IntPtr hwnd;
                private int message;
                private IntPtr wParam;
                private IntPtr lParam;
                private int time;
                [MarshalAs(UnmanagedType.ByValArray, SizeConst = 2)]
                private int[] pt;

                internal MSG()
                {
                    this.hwnd = IntPtr.Zero;
                    this.message = default(int);
                    this.wParam = IntPtr.Zero;
                    this.lParam = IntPtr.Zero;
                    this.time = default(int);
                    this.pt = new int[2];
                }

                internal int Message
                {
                    get
                    {
                        return this.message;
                    }
                }

                internal int Time
                {
                    get
                    {
                        return this.time;
                    }
                }
            }

            [DllImport("coredll.dll", SetLastError = false)]
            internal static extern short GetAsyncKeyState(int vKey);

            [DllImport("coredll.dll", SetLastError = false)]
            internal static extern int PeekMessage(
                MSG lpMsg,
                IntPtr hWnd,
                int wMsgFilterMin,
                int wMsgFilterMax,
                int wRemoveMsg);
        }

         #endregion
    }
    
}
